import "core-js/modules/web.dom-collections.iterator";

/**
two way to update state

1. action("push|replace")->historyapi("push|replace")->historychangelistener->dispatch->updateState

2. historychange -> historychangelistener->dispatch->updateState

*/
import { URL_CHANGE, URL_CHANGE_BLOCK, URL_CHANGE_UNBLOCK } from './constants';
import { queryStringToJSON, jsonToQueryString } from './utils';
import { routing } from './reducer';
const actions = {
  push: 'push',
  replace: 'replace',
  goback: 'goBack'
};
export const reduxRouter = (history, urls, onUrlChange, isBatchDispatch = false, loadPromise = null) => {
  let routingReducer = routing(urls);
  let initialUrlResolved;
  let initialUrlReject;
  let initialUrlDispatchedPromise = new Promise((res, rej) => {
    initialUrlResolved = res;
    initialUrlReject = rej;
  });

  function historyMiddleware(history) {
    return store => {
      let preUrlChangeReject = null;
      let isURLChanging = false;
      let storeActions = [];

      let onChange = (location, action) => {
        isURLChanging = true;
        let {
          search
        } = location;

        if (search) {
          location.query = queryStringToJSON(search);
        }

        let newRoutingState = routingReducer(store.getState().routing, {
          type: URL_CHANGE,
          from: 'history',
          data: {
            location
          }
        });

        if (preUrlChangeReject) {
          store.dispatch({
            type: 'URL_CHANGE_Failure',
            from: 'history',
            data: {
              location,
              error: {
                message: 'previous URL change failure'
              }
            }
          });
          storeActions = [];
          preUrlChangeReject = null;
        }

        onUrlChange(newRoutingState, () => {
          let promises = [];
          preUrlChangeReject = null;
          isURLChanging = false;

          if (isBatchDispatch) {
            promises.push(store.dispatch(storeActions));
          } else {
            storeActions.forEach(action => {
              promises.push(store.dispatch(action));
            });
          }

          return Promise.all(promises).then(() => {
            storeActions = [];
            let promise = store.dispatch({
              type: URL_CHANGE,
              from: 'history',
              data: {
                location
              }
            });

            if (loadPromise) {
              return promise.then(() => {
                initialUrlResolved && initialUrlResolved();
                initialUrlResolved = null;
                return Promise.resolve();
              });
            } else {
              return Promise.resolve();
            }
          });
        }, {
          dispatch: store.dispatch,
          getState: store.getState
        }, e => {
          isURLChanging = false;
          preUrlChangeReject = true;
          store.dispatch({
            type: 'URL_CHANGE_Failure',
            from: 'history',
            data: {
              location,
              error: e
            }
          });
          return Promise.reject();
        });
      };
      /* init dispatch*/


      if (loadPromise) {
        loadPromise.then(() => {
          try {
            onChange(history.getCurrentLocation ? history.getCurrentLocation() : history.location);
            /* setTimeout(() => {
              initialUrlResolved();
            }, 1) */
          } catch (e) {
            initialUrlReject(e);
          }
        });
      } else {
        setTimeout(() => {
          try {
            onChange(history.getCurrentLocation ? history.getCurrentLocation() : history.location);
            initialUrlResolved();
          } catch (e) {
            initialUrlReject(e);
          }
        }, 1);
      }
      /*  */

      /*store.dispatch({
        type:URL_CHANGE,
        from:"history",data:{
          location:history.getCurrentLocation ? history.getCurrentLocation() : history.location // only works history ^3.0.0
        }
      })*/


      let unlisten = history.listen(onChange);
      let unblock = null;
      return next => action => {
        if (action.type === URL_CHANGE && action.from === 'history') {
          if (unblock) {
            unblock();
            unblock = null;
            next({
              type: URL_CHANGE_UNBLOCK
            });
          }

          return next(Object.assign(action, {
            from: null
          })); //from is a flag used for update url only via history api
        } else if (action.type === URL_CHANGE) {
          if (action.data.location.action === 'POP') {
            history.go(-1);
          } else {
            if (action.data.location.href) {
              history[actions[action.data.location.action.toLowerCase()]](action.data.location.href);
            } else {
              let location = Object.assign({}, action.data.location);
              let routing = store.getState().routing || {};
              let {
                query = routing.query
              } = location;
              location.search = jsonToQueryString(query || {});
              history[actions[action.data.location.action.toLowerCase()]](location);
            }
          }
        } else if (action.type === URL_CHANGE_BLOCK) {
          unblock = history.block(action.data.msg || 'Are you sure to leave the page?'); //i18n miss

          next(action);
        } else if (action.type === URL_CHANGE_UNBLOCK) {
          if (unblock) {
            unblock();
            unblock = null;
          }

          next(action);
        } else if (isURLChanging) {
          storeActions.push(action);
          return action;
        } else {
          return next(action);
        }
      };
    };
  }

  return {
    historyMiddleware: historyMiddleware(history),
    routing: routingReducer,
    isRouterLoaded: () => initialUrlDispatchedPromise
  };
};
export const historyMiddleware = history => store => {
  /* init dispatch*/
  let location = history.getCurrentLocation ? history.getCurrentLocation() : history.location;
  /* only works history ^3.0.0 */

  let search = location.search;

  if (search) {
    location.query = queryStringToJSON(search);
  }

  store.dispatch({
    type: URL_CHANGE,
    from: 'history',
    data: {
      location: location
    }
  });
  let unlisten = history.listen((location, action) => {
    let {
      search
    } = location;

    if (search) {
      location.query = queryStringToJSON(search);
    }

    store.dispatch({
      type: URL_CHANGE,
      from: 'history',
      data: {
        location
      }
    });
  });
  let unblock = null;
  return next => action => {
    if (action.type == URL_CHANGE && action.from == 'history') {
      if (unblock) {
        unblock();
        unblock = null;
        next({
          type: URL_CHANGE_UNBLOCK
        });
      }

      return next(Object.assign(action, {
        from: null
      })); //from is a flag used for update url only via history api
    } else if (action.type == URL_CHANGE) {
      if (action.data.location.action == 'POP') {
        history.go(-1);
      } else {
        if (action.data.location.href) {
          history[actions[action.data.location.action.toLowerCase()]](action.data.location.href);
        } else {
          let location = Object.assign({}, action.data.location);
          let query = location.query;
          location.search = jsonToQueryString(location.query || {});
          history[actions[action.data.location.action.toLowerCase()]](location);
        }
      }
    } else if (action.type == URL_CHANGE_BLOCK) {
      unblock = history.block(action.data.msg || 'Are you sure to leave the page?'); //i18n miss

      next(action);
    } else if (action.type == URL_CHANGE_UNBLOCK) {
      if (unblock) {
        unblock();
        unblock = null;
      }

      next(action);
    } else {
      return next(action);
    }
  };
};