"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.historyMiddleware = exports.reduxRouter = void 0;

var _constants = require("./constants");

var _utils = require("./utils");

var _reducer = require("./reducer");

/**
two way to update state

1. action("push|replace")->historyapi("push|replace")->historychangelistener->dispatch->updateState

2. historychange -> historychangelistener->dispatch->updateState

*/
var actions = {
  push: 'push',
  replace: 'replace',
  goback: 'goBack'
};

var reduxRouter = function reduxRouter(history, urls, onUrlChange) {
  var isBatchDispatch = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : false;
  var loadPromise = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : null;
  var routingReducer = (0, _reducer.routing)(urls);
  var initialUrlResolved;
  var initialUrlReject;
  var initialUrlDispatchedPromise = new Promise(function (res, rej) {
    initialUrlResolved = res;
    initialUrlReject = rej;
  });

  function historyMiddleware(history) {
    return function (store) {
      var preUrlChangeReject = null;
      var isURLChanging = false;
      var storeActions = [];

      var onChange = function onChange(location, action) {
        isURLChanging = true;
        var search = location.search;

        if (search) {
          location.query = (0, _utils.queryStringToJSON)(search);
        }

        var newRoutingState = routingReducer(store.getState().routing, {
          type: _constants.URL_CHANGE,
          from: 'history',
          data: {
            location: location
          }
        });

        if (preUrlChangeReject) {
          store.dispatch({
            type: 'URL_CHANGE_Failure',
            from: 'history',
            data: {
              location: location,
              error: {
                message: 'previous URL change failure'
              }
            }
          });
          storeActions = [];
          preUrlChangeReject = null;
        }

        onUrlChange(newRoutingState, function () {
          var promises = [];
          preUrlChangeReject = null;
          isURLChanging = false;

          if (isBatchDispatch) {
            promises.push(store.dispatch(storeActions));
          } else {
            storeActions.forEach(function (action) {
              promises.push(store.dispatch(action));
            });
          }

          return Promise.all(promises).then(function () {
            storeActions = [];
            var promise = store.dispatch({
              type: _constants.URL_CHANGE,
              from: 'history',
              data: {
                location: location
              }
            });

            if (loadPromise) {
              return promise.then(function () {
                initialUrlResolved && initialUrlResolved();
                initialUrlResolved = null;
                return Promise.resolve();
              });
            } else {
              return Promise.resolve();
            }
          });
        }, {
          dispatch: store.dispatch,
          getState: store.getState
        }, function (e) {
          isURLChanging = false;
          preUrlChangeReject = true;
          store.dispatch({
            type: 'URL_CHANGE_Failure',
            from: 'history',
            data: {
              location: location,
              error: e
            }
          });
          return Promise.reject();
        });
      };
      /* init dispatch*/


      if (loadPromise) {
        loadPromise.then(function () {
          try {
            onChange(history.getCurrentLocation ? history.getCurrentLocation() : history.location);
            /* setTimeout(() => {
              initialUrlResolved();
            }, 1) */
          } catch (e) {
            initialUrlReject(e);
          }
        });
      } else {
        setTimeout(function () {
          try {
            onChange(history.getCurrentLocation ? history.getCurrentLocation() : history.location);
            initialUrlResolved();
          } catch (e) {
            initialUrlReject(e);
          }
        }, 1);
      }
      /*  */

      /*store.dispatch({
        type:URL_CHANGE,
        from:"history",data:{
          location:history.getCurrentLocation ? history.getCurrentLocation() : history.location // only works history ^3.0.0
        }
      })*/


      var unlisten = history.listen(onChange);
      var unblock = null;
      return function (next) {
        return function (action) {
          if (action.type === _constants.URL_CHANGE && action.from === 'history') {
            if (unblock) {
              unblock();
              unblock = null;
              next({
                type: _constants.URL_CHANGE_UNBLOCK
              });
            }

            return next(Object.assign(action, {
              from: null
            })); //from is a flag used for update url only via history api
          } else if (action.type === _constants.URL_CHANGE) {
            if (action.data.location.action === 'POP') {
              history.go(-1);
            } else {
              if (action.data.location.href) {
                history[actions[action.data.location.action.toLowerCase()]](action.data.location.href);
              } else {
                var location = Object.assign({}, action.data.location);

                var _routing = store.getState().routing || {};

                var _location$query = location.query,
                    query = _location$query === void 0 ? _routing.query : _location$query;
                location.search = (0, _utils.jsonToQueryString)(query || {});
                history[actions[action.data.location.action.toLowerCase()]](location);
              }
            }
          } else if (action.type === _constants.URL_CHANGE_BLOCK) {
            unblock = history.block(action.data.msg || 'Are you sure to leave the page?'); //i18n miss

            next(action);
          } else if (action.type === _constants.URL_CHANGE_UNBLOCK) {
            if (unblock) {
              unblock();
              unblock = null;
            }

            next(action);
          } else if (isURLChanging) {
            storeActions.push(action);
            return action;
          } else {
            return next(action);
          }
        };
      };
    };
  }

  return {
    historyMiddleware: historyMiddleware(history),
    routing: routingReducer,
    isRouterLoaded: function isRouterLoaded() {
      return initialUrlDispatchedPromise;
    }
  };
};

exports.reduxRouter = reduxRouter;

var historyMiddleware = function historyMiddleware(history) {
  return function (store) {
    /* init dispatch*/
    var location = history.getCurrentLocation ? history.getCurrentLocation() : history.location;
    /* only works history ^3.0.0 */

    var search = location.search;

    if (search) {
      location.query = (0, _utils.queryStringToJSON)(search);
    }

    store.dispatch({
      type: _constants.URL_CHANGE,
      from: 'history',
      data: {
        location: location
      }
    });
    var unlisten = history.listen(function (location, action) {
      var search = location.search;

      if (search) {
        location.query = (0, _utils.queryStringToJSON)(search);
      }

      store.dispatch({
        type: _constants.URL_CHANGE,
        from: 'history',
        data: {
          location: location
        }
      });
    });
    var unblock = null;
    return function (next) {
      return function (action) {
        if (action.type == _constants.URL_CHANGE && action.from == 'history') {
          if (unblock) {
            unblock();
            unblock = null;
            next({
              type: _constants.URL_CHANGE_UNBLOCK
            });
          }

          return next(Object.assign(action, {
            from: null
          })); //from is a flag used for update url only via history api
        } else if (action.type == _constants.URL_CHANGE) {
          if (action.data.location.action == 'POP') {
            history.go(-1);
          } else {
            if (action.data.location.href) {
              history[actions[action.data.location.action.toLowerCase()]](action.data.location.href);
            } else {
              var _location = Object.assign({}, action.data.location);

              var query = _location.query;
              _location.search = (0, _utils.jsonToQueryString)(_location.query || {});
              history[actions[action.data.location.action.toLowerCase()]](_location);
            }
          }
        } else if (action.type == _constants.URL_CHANGE_BLOCK) {
          unblock = history.block(action.data.msg || 'Are you sure to leave the page?'); //i18n miss

          next(action);
        } else if (action.type == _constants.URL_CHANGE_UNBLOCK) {
          if (unblock) {
            unblock();
            unblock = null;
          }

          next(action);
        } else {
          return next(action);
        }
      };
    };
  };
};

exports.historyMiddleware = historyMiddleware;