"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.routing = void 0;

var _normalizer = require("@zohodesk/normalizer");

var _constants = require("./constants");

var _utils = require("./utils");

/*
	routing:{
		location:{
		  pathname:,
		  state,
		  query,
		},
		urls:{
		  entity:{
		    "url1":{
		      name:
		      pattern:
		      match:
		      tokens:
		    }
		  },
		  order:["url1",..],
		},
		currentParams:{

		}
	}
  [
    {
      name:

    }
    {
      name:"ticketList",
      pattern:"/zd/:portalId/tickets",
      parent:"ticket"
    }
  ]
*/
var routing = function routing(urls) {
  var urlNormalizer = (0, _normalizer.normalize)(urls, _utils.urlsSchema);
  var urls = (0, _utils.initialStateWithTokens)(urlNormalizer.entities.urls);
  var order = urlNormalizer.result;
  var initialState = {
    urls: urls,
    order: order
  };
  return function () {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : initialState;
    var action = arguments.length > 1 ? arguments[1] : undefined;
    var currentParams;

    switch (action.type) {
      case _constants.URL_CHANGE:
        var flag = 1;
        var page;

        var _urls = state.order.reduce(function (result, next) {
          var obj = (0, _utils.matchPattern)(state.urls[next].pattern, action.data.location.pathname);
          var match = _constants.NOT_MATCH;

          if (obj) {
            if (flag == 1) {
              currentParams = obj;
            }

            if (obj.remainingPathname == "" && flag == 1) {
              match = _constants.EXACT_MATCH;
              page = next;
              flag = 0;
            } else if (flag == 1) {
              match = _constants.MATCH;
            }
          }

          result[next] = url(state.urls[next], action, match);
          return result;
        }, {});

        return {
          /* have to optimize code - skip all process once exact match*/
          urls: _urls,
          page: page,
          order: state.order,
          paramMap: currentParams && currentParams.paramMap || {},
          location: location(state.location, action) //old location and new location are diff deep compare

        };

      case _constants.URL_CHANGE_BLOCK:
        return Object.assign({}, state, {
          block: true,
          blockReason: action.data.msg
        });

      case _constants.URL_CHANGE_UNBLOCK:
        return Object.assign({}, state, {
          block: false,
          blockReason: null
        });
    }

    return state;
  };
};

exports.routing = routing;

var url = function url() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var action = arguments.length > 1 ? arguments[1] : undefined;
  var matchType = arguments.length > 2 ? arguments[2] : undefined;
  return Object.assign({}, state, {
    match: matchType
  });
};

var location = function location() {
  var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var action = arguments.length > 1 ? arguments[1] : undefined;

  if (action.type == _constants.URL_CHANGE) {
    return action.data.location;
  }

  return state;
};